#include <qtextbrowser.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include <qobjectlist.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qaction.h>
#include <qtimer.h>

#include <qpe/qpeapplication.h>
#include <qpe/config.h>

#include "mainwindow.h"
#include "tabledata.h"
#include "details.h"
#include "defines.h"

static TableEntry header_;

ElementButton::ElementButton(TableEntry &entry, DisplayMode &mode, bool color, QWidget *parent)
	: QPushButton(parent), _entry(entry)
{
	setGeometry((entry.x - 1) * 30 + 5, (entry.y - 1) * 25 + 5, 25, 20);

	if (mode == Symbols)
		setText(entry.info[TableEntry::SYMBOL]);
	else
		setText(entry.info[TableEntry::NUMBER]);

	if (color) {
		QPalette p = palette();
		QColor c = QColor(entry.red, entry.green, entry.blue);
		p.setColor(QColorGroup::Button, c);
		p.setColor(QColorGroup::ButtonText, QColor("#000000"));
		setPalette(p);
		setBackgroundMode(QWidget::PaletteBackground);
	}

	connect(this, SIGNAL(clicked()), SLOT(slotClicked()));
}

void ElementButton::slotClicked()
{
	Details *details = new Details(_entry, header_, 0, 0, Qt::WType_Modal);
	details->showMaximized();
	details->show();
}

MainWindow::MainWindow(QWidget *parent, const char *name, WFlags fl)
	: MainWindowBase(parent, name, fl)
{
	_fieldWidth = 0;
	_fieldHeight = 0;

	_config = new Config("qperiodic");
	_config->setGroup("Settings");

	bool cColor = _config->readBoolEntry("Color", true);
	bool cSymbols = _config->readBoolEntry("Symbols", true);
	color->setOn(cColor);
	_color = cColor;
	cSymbols ? dispSymbol->setOn(true) : dispNumber->setOn(true);
	_dispMode = cSymbols ? ElementButton::Symbols : ElementButton::Numbers;

	connect(color, SIGNAL(activated()), SLOT(slotColor()));
	connect(dispSymbol, SIGNAL(activated()), SLOT(slotDispSymbols()));
	connect(dispNumber, SIGNAL(activated()), SLOT(slotDispNumbers()));
	connect(about, SIGNAL(activated()), SLOT(slotAbout()));
	connect(quit, SIGNAL(activated()), SLOT(close()));

	drawField();
}

MainWindow::~MainWindow()
{
	_config->setGroup("Settings");
	_config->writeEntry("Color", color->isOn());
	_config->writeEntry("Symbols", (_dispMode == ElementButton::Symbols) ? true : false);
	_config->write();
}

void MainWindow::keyReleaseEvent(QKeyEvent *event)
{
	if (event->key() == Qt::Key_F11) { // Zaurus Menubutton
		QSize sz(menuButton->popup()->sizeHint());
		QPoint p = menuButton->mapToGlobal(menuButton->rect().topLeft());
		p.ry() -= sz.height();
		menuButton->popup()->exec(p);
	}

	MainWindowBase::keyReleaseEvent(event);
}

void MainWindow::drawField()
{
//	if (_field != NULL)
//		delete _field;

	_scrollField = new QWidget(this);
	QGridLayout *layout = new QGridLayout(_scrollField);

	QSpacerItem *item1 = new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	layout->addItem(item1, 0, 0);

	_field = new QWidget(_scrollField);
	layout->addWidget(_field, 1, 0);

	QSpacerItem *item2 = new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	layout->addItem(item2, 2, 0);

	TableData *table = new TableData();
	header_ = table->header();
	TableEntry *data = table->data();

	for (int i = 0; i <= ELEMENTS - 1; i++) {
		if (data[i].x > _fieldWidth) _fieldWidth = data[i].x;
		if (data[i].y > _fieldHeight) _fieldHeight = data[i].y;

		(void) new ElementButton(data[i], _dispMode, _color, _field);
	}
	_fieldWidth = _fieldWidth * 30 + 5;
	_fieldHeight = _fieldHeight * 25 + 5;

	_field->setMinimumSize(_fieldWidth, _fieldHeight);
	addChild(_scrollField);

	QTimer::singleShot(0, this, SLOT(slotResizeField()));
}

void MainWindow::slotResizeField()
{
	_scrollField->resize(
		(_fieldWidth < visibleWidth())?visibleWidth():_fieldWidth, 
		(_fieldHeight < visibleHeight())?visibleHeight():_fieldHeight); 
}

void MainWindow::slotColor()
{
	_color = !_color;
	QMessageBox::information(this, tr("Notice"), tr("<p>You have to restart this application to let the changes take effect."), tr("OK"));
}

void MainWindow::slotDispSymbols()
{
	_dispMode = ElementButton::Symbols;
	QMessageBox::information(this, tr("Notice"), tr("<p>You have to restart this application to let the changes take effect."), tr("OK"));
}

void MainWindow::slotDispNumbers()
{
	_dispMode = ElementButton::Numbers;
	QMessageBox::information(this, tr("Notice"), tr("<p>You have to restart this application to let the changes take effect."), tr("OK"));
}

void MainWindow::slotAbout()
{
	QString aboutHtml = tr(
		"<html><body>"
		"<p align=center><img src=%1><br>"
		"<b>QPeriodic</b><br>"
		"<i>A Periodic Table for Qtopia/Opie</i></p><br>"
		"<p><b>Version:</b> %2</p>"
		"<p><b>Thanks to:</b> The GPeriodic developement team for "
		"putting together a great Periodic Table. I took the data "
		"from them. Thanks very much!</p>"
		"<p><b>License:</b> This program is licensed under the terms "
		"of the GNU General Public License 2.0</p><br><hr>"
		"Constantin Bergemann<br>"
		"<i>cbergemann@lisa.de</i>"
		"</body></html>")
		.arg(QPEApplication::qpeDir() + "/pics/qperiodic.png")
		.arg(VERSION);

	QTextBrowser *aboutWidget = new QTextBrowser();
	aboutWidget->showMaximized();
	aboutWidget->setCaption(tr("About QPeriodic"));
	aboutWidget->setText(aboutHtml);
	aboutWidget->show();
}

